//---------------------------------------------------------------
//---------------------------------------------------------------
//------------  Simple Genetic Algorithm             ------------
//------------  - binary representation              ------------  
//------------  - generational replacement strategy  ------------  
//---------------------------------------------------------------
//---------------------------------------------------------------
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <conio.h>
#include <time.h>
#include <string.h>
#include <float.h>
//----------------
#include "general_sga.h"
#include "individual.h"
#include "generation_sga.h"

tind *newpop, *backpop, *offspring; // old and new population
//------- SGA parameters
int nTournament;		// parameter of tournament selection
int pCross;				// probability of crossover [%]
int pMutation;			// probability of mutation [%]

//------------------------------------------------------
//------- SGA functions
//------------------------------------------------------

//------- Population statistics
void statistics(){
	int i;
	double pom=0.0;

	//------- init statdata
	statdata.fbest = -1.0e100;
	statdata.fworst = 1.0e100;
	statdata.favg = 0.0;
	statdata.fsum = 0.0;
	statdata.indbest = 0;
	statdata.indworst = 0;
	for(i=0; i<popSize; i++){
		if(statdata.fbest <= population[i].fitness){
			statdata.fbest = population[i].fitness;
			statdata.indbest = i;
		}
		if(statdata.fworst >= population[i].fitness){
			statdata.fworst = population[i].fitness;
			statdata.indworst = i;
		}
		statdata.fsum += population[i].fitness;
	}
	statdata.favg = statdata.fsum / popSize;
}

//----------------------------------------------------------------------
//------- copies the individual from pop_from to pop_to
void copy_ind(int ind_from,tind *pop_from,int ind_to,tind *pop_to){
  int i;

  for(i=0; i<maxGenes; i++){
	  pop_to[ind_to].genes[i] = pop_from[ind_from].genes[i];
  }
  pop_to[ind_to].fitness = pop_from[ind_from].fitness;
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
//------ Tournament selection                   
//------   - tournamentSize individuals take part in tournament
int select_tournament(tind *pop, int tournamentSize){
	int cand, tempbest;

	//------- 1st competitor
	tempbest = rand() % popSize; //--- best-so-far candidate
	tournamentSize--;

	//------- try remaining competitors
	while(tournamentSize > 0){ 
		cand = rand() % popSize; 
		if(pop[cand].fitness > pop[tempbest].fitness)
			tempbest = cand;
		//-------
		tournamentSize--;
	}
	return tempbest;
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
//------ Chooses parents using select() strategy                   
void choose_parents(tind *pop, int *par1, int *par2){

	//------- 1st parent
	*par1 = select_tournament(pop, nTournament);

	//------- chooses 2nd parent to be different than the 1st one
	do{
		*par2 = select_tournament(pop, nTournament);
	}while(*par1 == *par2);
}

//------- Initializes population with given index popId, population size, crossover rate 
//------- and the tournament selection parameter.
//------- Returns 1 if the initialization went OK; otherwise returns 0.
int initPopulation(int *evals){
	int i, j;

	//--- allocates memory for chromosomes
	if((population = (tind *)malloc(popSize * sizeof(tind))) == NULL){
		printf("\nNot enough memory.");
		return 0;	//--- not enough memory
	}
	for(i=0; i<popSize; i++){
		if((population[i].genes = (int *)malloc(maxGenes * sizeof(int))) == NULL){
			printf("\nNot enough memory.");
			return 0;	//--- not enough memory
		}
	}

	//--- initializes chromosomes
	//--- randomly generates chromosomes with the probability of generating 1 of P(1)=0.5 
	for(i=0; i<popSize; i++){ 
		for(j=0; j<maxGenes; j++){
			if(rand()%100 < 50){              
				population[i].genes[j] = 1;
			}
			else{
				population[i].genes[j] = 0;
			}
		}
	}

	//------- evaluate chromosomes
	for(i=0; i<popSize; i++){
		get_fitness(&(population[i]));
		(*evals)++;
	}
	//------- calculate population statistics
	statistics();

	printf("\n*** Population %d created:\t fitness=%.3f (%.3f)", popSize, statdata.favg, statdata.fbest);

	return 1;	//--- everything is OK
}

//------- Destroys population with given index popId
void destroyPopulation(){
	int i,j;

	//--- releases memory allocated to chromosomes of the given population
	for(i=0; i<popSize; i++)
		free(population[i].genes);
	free(population);
	population = NULL;
}

//------- allocate memory for new population and offspring
void allocateMemory(){
	int i;

	newpop = (tind *)malloc(popSize * sizeof(tind));
	for(i=0; i<popSize; i++){
		newpop[i].genes = (int *)malloc(maxGenes * sizeof(int));
	}
	offspring = (tind *)malloc(2 * sizeof(tind));
	for(i=0; i<2; i++){
		offspring[i].genes = (int *)malloc(maxGenes * sizeof(int));
	}
}

void freeMemory(){
	int i;

	//------- free memory allocated for the new population and offspring
	for(i=0; i<popSize; i++){
		free(newpop[i].genes);
	}
	free(newpop);
	for(i=0; i<2; i++){
		free(offspring[i].genes);
	}
	free(offspring);
}

//------- Performs one generation with the population popId
void doGeneration(int *evals){
	int par1, par2;         // indexes of parents
	int chroms_generated;   // a number of newly generated chromosomes

	//------- allocate memory for new population and offspring
	allocateMemory();

	//------- elitism
	chroms_generated = 0;
	copy_ind(statdata.indbest, population, chroms_generated++, newpop);

	//------- create new population of chromosomes (generational replacement strategy)
	do{
		choose_parents(population, &par1, &par2);	// choose parents
		//------- create two new chromosomes
		if(rand()%100 < pCross){
			crossover(population[par1].genes, population[par2].genes, offspring[0].genes, offspring[1].genes);
		}
		else{
			copy_ind(par1, population, 0, offspring);
			copy_ind(par2, population, 1, offspring);
			mutation(offspring[0].genes);
			mutation(offspring[1].genes);
		}
		//------- evaluate offspring
		get_fitness(&(offspring[0])); (*evals)++;
		get_fitness(&(offspring[1])); (*evals)++;
		//------- replacement
		copy_ind(0, offspring, chroms_generated++, newpop);
		if(chroms_generated < popSize)
			copy_ind(1, offspring, chroms_generated++, newpop);
	}while(chroms_generated < popSize);

	//------- switch populations
	backpop = population;
	population = newpop;
	newpop = backpop;
	statistics();		//------- calculate population statistics

	//------- free memory allocated for the new population and offspring
	freeMemory();

	printf("\n  - Evaluations %d:\t fitness=%.3f (%.3f)", *evals, statdata.favg, statdata.fbest);
}




